from __future__ import annotations

import copy
from typing import cast

import pytest

from iaqualink.systems.exo.device import (
    EXO_TEMP_CELSIUS_HIGH,
    EXO_TEMP_CELSIUS_LOW,
    ExoAttributeSensor,
    ExoAttributeSwitch,
    ExoAuxSwitch,
    ExoDevice,
    ExoSensor,
    ExoSwitch,
    ExoThermostat,
)
from iaqualink.systems.exo.system import ExoSystem

from ...base_test_device import (
    TestBaseDevice,
    TestBaseSensor,
    TestBaseSwitch,
    TestBaseThermostat,
)


class TestExoDevice(TestBaseDevice):
    def setUp(self) -> None:
        super().setUp()

        data = {"serial_number": "SN123456", "device_type": "exo"}
        self.system = ExoSystem(self.client, data=data)

        data = {"name": "Test Device", "state": "42"}
        self.sut = ExoDevice(self.system, data)
        self.sut_class = ExoDevice

    def test_equal(self) -> None:
        assert self.sut == self.sut

    def test_not_equal(self) -> None:
        obj2 = copy.deepcopy(self.sut)
        obj2.data["name"] = "Test Device 2"
        assert self.sut != obj2

    def test_property_name(self) -> None:
        assert self.sut.name == self.sut.data["name"]

    def test_property_state(self) -> None:
        assert self.sut.state == str(self.sut.data["state"])

    def test_not_equal_different_type(self) -> None:
        assert (self.sut == {}) is False

    def test_property_manufacturer(self) -> None:
        assert self.sut.manufacturer == "Zodiac"

    def test_property_model(self) -> None:
        assert self.sut.model == self.sut_class.__name__.replace("Exo", "")


class TestExoSensor(TestExoDevice, TestBaseSensor):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "name": "sns_1",
            "sensor_type": "Foo",
            "value": 42,
            "state": 1,
        }
        self.sut = ExoDevice.from_data(self.system, data)
        self.sut_class = ExoSensor

    def test_property_name(self) -> None:
        assert self.sut.name == self.sut.data["sensor_type"].lower().replace(
            " ", "_"
        )

    def test_property_state(self) -> None:
        assert self.sut.state == str(self.sut.data["value"])


class TestExoAttributeSensor(TestExoDevice):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "name": "foo_bar",
            "state": 42,
        }
        self.sut = ExoDevice.from_data(self.system, data)
        self.sut_class = ExoAttributeSensor


class ExoSwitchMixin:
    def test_property_is_on_false(self) -> None:
        self.sut.data["state"] = 0
        super().test_property_is_on_false()
        assert self.sut.is_on is False

    def test_property_is_on_true(self) -> None:
        self.sut.data["state"] = 1
        super().test_property_is_on_true()
        assert self.sut.is_on is True


class TestExoSwitch(TestExoDevice, ExoSwitchMixin, TestBaseSwitch):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "name": "toggle",
            "state": 0,
        }

        # ExoSwitch is an abstract class, not meant to be instantiated directly.
        self.sut = ExoSwitch(self.system, data)
        self.sut_class = ExoSwitch

    async def test_turn_on(self) -> None:
        self.sut.data["state"] = 0
        with pytest.raises(NotImplementedError):
            await super().test_turn_on()

    async def test_turn_on_noop(self) -> None:
        self.sut.data["state"] = 1
        await super().test_turn_on_noop()

    async def test_turn_off(self) -> None:
        self.sut.data["state"] = 1
        with pytest.raises(NotImplementedError):
            await super().test_turn_off()

    async def test_turn_off_noop(self) -> None:
        self.sut.data["state"] = 0
        await super().test_turn_off_noop()


class TestExoAuxSwitch(TestExoDevice, ExoSwitchMixin, TestBaseSwitch):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "name": "aux_1",
            "type": "Foo",
            "mode": "mode",
            "light": 0,
            "state": 1,
        }
        self.sut = ExoDevice.from_data(self.system, data)
        self.sut_class = ExoAuxSwitch

    async def test_turn_on(self) -> None:
        self.sut.data["state"] = 0
        await super().test_turn_on()

    async def test_turn_on_noop(self) -> None:
        self.sut.data["state"] = 1
        await super().test_turn_on_noop()

    async def test_turn_off(self) -> None:
        self.sut.data["state"] = 1
        await super().test_turn_off()

    async def test_turn_off_noop(self) -> None:
        self.sut.data["state"] = 0
        await super().test_turn_off_noop()


class TestExoAttributeSwitch(TestExoDevice, ExoSwitchMixin, TestBaseSwitch):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "name": "boost",
            "state": 1,
        }
        self.sut = ExoDevice.from_data(self.system, data)
        self.sut_class = ExoAttributeSwitch

    async def test_turn_on(self) -> None:
        self.sut.data["state"] = 0
        await super().test_turn_on()

    async def test_turn_on_noop(self) -> None:
        self.sut.data["state"] = 1
        await super().test_turn_on_noop()

    async def test_turn_off(self) -> None:
        self.sut.data["state"] = 1
        await super().test_turn_off()

    async def test_turn_off_noop(self) -> None:
        self.sut.data["state"] = 0
        await super().test_turn_off_noop()


class TestExoThermostat(TestExoDevice, TestBaseThermostat):
    def setUp(self) -> None:
        super().setUp()

        pool_set_point = {
            "name": "heating",
            "enabled": 1,
            "sp": 20,
            "sp_min": 1,
            "sp_max": 40,
        }

        self.pool_set_point = cast(
            ExoThermostat, ExoDevice.from_data(self.system, pool_set_point)
        )

        water_temp = {
            "name": "sns_3",
            "sensor_type": "Water Temp",
            "state": 1,
            "value": 16,
        }
        self.water_temp = ExoDevice.from_data(self.system, water_temp)

        devices = [
            self.pool_set_point,
            self.water_temp,
        ]
        self.system.devices = {x.data["name"]: x for x in devices}

        self.sut = self.pool_set_point
        self.sut_class = ExoThermostat

    def test_property_label(self) -> None:
        assert self.sut.label == "Heating"

    def test_property_name(self) -> None:
        assert self.sut.name == "heating"

    def test_property_state(self) -> None:
        assert self.sut.state == "20"

    def test_property_is_on_true(self) -> None:
        self.sut.data["enabled"] = 1
        super().test_property_is_on_true()

    def test_property_is_on_false(self) -> None:
        self.sut.data["enabled"] = 0
        super().test_property_is_on_false()

    def test_property_unit(self) -> None:
        assert self.sut.unit == "C"

    @pytest.mark.skip(reason="Exo doesn't support Fahrenheit")
    def test_property_min_temperature_f(self) -> None:
        pass

    def test_property_min_temperature_c(self) -> None:
        self.sut.system.temp_unit = "C"
        super().test_property_min_temperature_c()
        assert self.sut.min_temperature == EXO_TEMP_CELSIUS_LOW

    @pytest.mark.skip(reason="Exo doesn't support Fahrenheit")
    def test_property_max_temperature_f(self) -> None:
        pass

    def test_property_max_temperature_c(self) -> None:
        self.sut.system.temp_unit = "C"
        super().test_property_max_temperature_c()
        assert self.sut.max_temperature == EXO_TEMP_CELSIUS_HIGH

    def test_property_current_temperature(self) -> None:
        super().test_property_current_temperature()
        assert self.sut.current_temperature == "16"

    def test_property_target_temperature(self) -> None:
        super().test_property_target_temperature()
        assert self.sut.target_temperature == "20"

    async def test_turn_on(self) -> None:
        self.sut.data["enabled"] = 0
        await super().test_turn_on()
        assert len(self.respx_calls) == 1
        content = self.respx_calls[0].request.content.decode("utf-8")
        assert "heating" in content

    async def test_turn_on_noop(self) -> None:
        self.sut.data["enabled"] = 1
        await super().test_turn_on_noop()

    async def test_turn_off(self) -> None:
        self.sut.data["enabled"] = 1
        await super().test_turn_off()
        assert len(self.respx_calls) == 1
        content = self.respx_calls[0].request.content.decode("utf-8")
        assert "heating" in content

    async def test_turn_off_noop(self) -> None:
        self.sut.data["enabled"] = 0
        await super().test_turn_off_noop()

    @pytest.mark.skip(reason="Exo doesn't support Fahrenheit")
    async def test_set_temperature_86f(self) -> None:
        pass

    async def test_set_temperature_30c(self) -> None:
        await super().test_set_temperature_30c()
        assert len(self.respx_calls) == 1
        content = self.respx_calls[0].request.content.decode("utf-8")
        assert "heating" in content



from __future__ import annotations

AQUALINK_API_KEY = "EOOEMOW4YR6QNB07"

AQUALINK_LOGIN_URL = "https://prod.zodiac-io.com/users/v1/login"
AQUALINK_DEVICES_URL = "https://r-api.iaqualink.net/devices.json"

KEEPALIVE_EXPIRY = 30
MIN_SECS_TO_REFRESH = 5



from __future__ import annotations

import copy
from unittest.mock import PropertyMock, patch

import pytest
import respx
import respx.router

from iaqualink.device import (
    AqualinkBinarySensor,
    AqualinkLight,
    AqualinkSensor,
    AqualinkSwitch,
    AqualinkThermostat,
)
from iaqualink.exception import (
    AqualinkInvalidParameterException,
    AqualinkOperationNotSupportedException,
)

from .base import TestBase, dotstar, resp_200


class TestBaseDevice(TestBase):
    def test_property_name(self) -> None:
        assert isinstance(self.sut.name, str)

    def test_property_label(self) -> None:
        assert isinstance(self.sut.label, str)

    def test_property_state(self) -> None:
        assert isinstance(self.sut.state, str)

    def test_property_manufacturer(self) -> None:
        assert isinstance(self.sut.manufacturer, str)

    def test_property_model(self) -> None:
        assert isinstance(self.sut.model, str)

    def test_from_data(self) -> None:
        if sut_class := getattr(self, "sut_class", None):
            assert isinstance(self.sut, sut_class)


class TestBaseSensor(TestBaseDevice):
    def test_inheritance(self) -> None:
        assert isinstance(self.sut, AqualinkSensor)


class TestBaseBinarySensor(TestBaseSensor):
    def test_inheritance(self) -> None:
        assert isinstance(self.sut, AqualinkBinarySensor)

    def test_property_is_on_true(self) -> None:
        assert self.sut.is_on is True

    def test_property_is_on_false(self) -> None:
        assert self.sut.is_on is False


class TestBaseSwitch(TestBaseBinarySensor):
    def test_inheritance(self) -> None:
        assert isinstance(self.sut, AqualinkSwitch)

    @respx.mock
    async def test_turn_on(self, respx_mock: respx.router.MockRouter) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.turn_on()
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_turn_on_noop(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.turn_on()
        assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_turn_off(self, respx_mock: respx.router.MockRouter) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.turn_off()
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_turn_off_noop(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.turn_off()
        assert len(respx_mock.calls) == 0


class TestBaseLight(TestBaseSwitch):
    def test_inheritance(self) -> None:
        assert isinstance(self.sut, AqualinkLight)

    def test_property_supports_brightness(self) -> None:
        assert isinstance(self.sut.supports_brightness, bool)

    def test_property_supports_effect(self) -> None:
        assert isinstance(self.sut.supports_effect, bool)

    def test_property_brightness(self) -> None:
        if not self.sut.supports_brightness:
            pytest.skip("Device doesn't support brightness")
        assert isinstance(self.sut.brightness, int)
        assert 0 <= self.sut.brightness <= 100

    def test_property_effect(self) -> None:
        if not self.sut.supports_effect:
            pytest.skip("Device doesn't support effects")
        assert isinstance(self.sut.effect, str)

    def test_property_supported_effects(self) -> None:
        if not self.sut.supports_effect:
            pytest.skip("Device doesn't support effects")
        assert isinstance(self.sut.supported_effects, dict)

    @respx.mock
    async def test_set_brightness_75(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_brightness:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_brightness(75)
            return

        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.set_brightness(75)
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_set_brightness_invalid_89(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_brightness:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_brightness(89)
            return

        respx_mock.route(dotstar).mock(resp_200)
        with pytest.raises(AqualinkInvalidParameterException):
            await self.sut.set_brightness(89)
        assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_set_effect_by_id_4(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_effect:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_effect_by_id(4)
            return

        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.set_effect_by_id(4)
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_set_effect_by_id_invalid_27(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_effect:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_effect_by_id(27)
            return

        respx_mock.route(dotstar).mock(resp_200)
        with pytest.raises(AqualinkInvalidParameterException):
            await self.sut.set_effect_by_id(27)
        assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_set_effect_by_name_off(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_effect:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_effect_by_name("Off")
            return

        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.set_effect_by_name("Off")
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_set_effect_by_name_invalid_amaranth(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        if not self.sut.supports_effect:
            with pytest.raises(AqualinkOperationNotSupportedException):
                await self.sut.set_effect_by_name("Amaranth")
            return

        respx_mock.route(dotstar).mock(resp_200)
        with pytest.raises(AqualinkInvalidParameterException):
            await self.sut.set_effect_by_name("Amaranth")
        assert len(respx_mock.calls) == 0


class TestBaseThermostat(TestBaseSwitch):
    def test_inheritance(self) -> None:
        assert isinstance(self.sut, AqualinkThermostat)

    def test_property_unit(self) -> None:
        assert self.sut.unit in ["C", "F"]

    def test_property_min_temperature_f(self) -> None:
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "F"
            assert isinstance(self.sut.min_temperature, int)

    def test_property_min_temperature_c(self) -> None:
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "C"
            assert isinstance(self.sut.min_temperature, int)

    def test_property_max_temperature_f(self) -> None:
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "F"
            assert isinstance(self.sut.max_temperature, int)

    def test_property_max_temperature_c(self) -> None:
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "C"
            assert isinstance(self.sut.max_temperature, int)

    def test_property_current_temperature(self) -> None:
        assert isinstance(self.sut.current_temperature, str)

    def test_property_target_temperature(self) -> None:
        assert isinstance(self.sut.target_temperature, str)

    @respx.mock
    async def test_set_temperature_86f(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "F"
            await self.sut.set_temperature(86)
            assert len(respx_mock.calls) > 0
            self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_set_temperature_30c(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "C"
            await self.sut.set_temperature(30)
            assert len(respx_mock.calls) > 0
            self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_set_temperature_invalid_400f(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "F"
            with pytest.raises(AqualinkInvalidParameterException):
                await self.sut.set_temperature(400)
            assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_set_temperature_invalid_204c(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        with patch.object(
            type(self.sut), "unit", new_callable=PropertyMock
        ) as mock_unit:
            mock_unit.return_value = "C"
            with pytest.raises(AqualinkInvalidParameterException):
                await self.sut.set_temperature(204)
            assert len(respx_mock.calls) == 0



from __future__ import annotations

import logging
from enum import Enum, unique
from typing import TYPE_CHECKING, cast

from iaqualink.device import (
    AqualinkBinarySensor,
    AqualinkDevice,
    AqualinkLight,
    AqualinkSensor,
    AqualinkSwitch,
    AqualinkThermostat,
)
from iaqualink.exception import (
    AqualinkDeviceNotSupported,
    AqualinkInvalidParameterException,
)

if TYPE_CHECKING:
    from iaqualink.systems.iaqua.system import IaquaSystem
    from iaqualink.typing import DeviceData

IAQUA_TEMP_CELSIUS_LOW = 1
IAQUA_TEMP_CELSIUS_HIGH = 40
IAQUA_TEMP_FAHRENHEIT_LOW = 34
IAQUA_TEMP_FAHRENHEIT_HIGH = 104

LOGGER = logging.getLogger("iaqualink")


@unique
class AqualinkState(Enum):
    OFF = "0"
    ON = "1"
    STANDBY = "2"
    ENABLED = "3"
    ABSENT = "absent"
    PRESENT = "present"


class IaquaDevice(AqualinkDevice):
    def __init__(self, system: IaquaSystem, data: DeviceData):
        super().__init__(system, data)

        # This silences mypy errors due to AqualinkDevice type annotations.
        self.system: IaquaSystem = system

    @property
    def label(self) -> str:
        if "label" in self.data:
            label = self.data["label"]
            return " ".join([x.capitalize() for x in label.split()])

        label = self.data["name"]
        return " ".join([x.capitalize() for x in label.split("_")])

    @property
    def state(self) -> str:
        return self.data["state"]

    @property
    def name(self) -> str:
        return self.data["name"]

    @property
    def manufacturer(self) -> str:
        return "Jandy"

    @property
    def model(self) -> str:
        return self.__class__.__name__.replace("Iaqua", "")

    @classmethod
    def from_data(cls, system: IaquaSystem, data: DeviceData) -> IaquaDevice:
        class_: type[IaquaDevice]

        # I don't have a system where these fields get populated.
        # No idea what they are and what to do with them.
        if isinstance(data["state"], dict | list):
            raise AqualinkDeviceNotSupported(data)

        if data["name"].endswith("_heater") or data["name"].endswith("_pump"):
            class_ = IaquaSwitch
        elif data["name"].endswith("_set_point"):
            if data["state"] == "":
                raise AqualinkDeviceNotSupported(data)
            class_ = IaquaThermostat
        elif data["name"] == "freeze_protection" or data["name"].endswith(
            "_present"
        ):
            class_ = IaquaBinarySensor
        elif data["name"].startswith("aux_"):
            if data["type"] == "2":
                class_ = light_subtype_to_class[data["subtype"]]
            elif data["type"] == "1":
                class_ = IaquaDimmableLight
            elif "LIGHT" in data["label"]:
                class_ = IaquaLightSwitch
            else:
                class_ = IaquaAuxSwitch
        else:
            class_ = IaquaSensor

        return class_(system, data)


class IaquaSensor(IaquaDevice, AqualinkSensor):
    pass


class IaquaBinarySensor(IaquaSensor, AqualinkBinarySensor):
    """These are non-actionable sensors, essentially read-only on/off."""

    @property
    def is_on(self) -> bool:
        return (
            AqualinkState(self.state)
            in [AqualinkState.ON, AqualinkState.ENABLED, AqualinkState.PRESENT]
            if self.state
            else False
        )


class IaquaSwitch(IaquaBinarySensor, AqualinkSwitch):
    async def _toggle(self) -> None:
        await self.system.set_switch(f"set_{self.name}")

    async def turn_on(self) -> None:
        if not self.is_on:
            await self._toggle()

    async def turn_off(self) -> None:
        if self.is_on:
            await self._toggle()


class IaquaAuxSwitch(IaquaSwitch):
    @property
    def is_on(self) -> bool:
        return (
            AqualinkState(self.state) == AqualinkState.ON
            if self.state
            else False
        )

    async def _toggle(self) -> None:
        await self.system.set_aux(self.data["aux"])


class IaquaLightSwitch(IaquaAuxSwitch, AqualinkLight):
    pass


class IaquaDimmableLight(IaquaAuxSwitch, AqualinkLight):
    async def turn_on(self) -> None:
        if not self.is_on:
            await self.set_brightness(100)

    async def turn_off(self) -> None:
        if self.is_on:
            await self.set_brightness(0)

    @property
    def brightness(self) -> int | None:
        return int(self.data["subtype"])

    async def set_brightness(self, brightness: int) -> None:
        # Brightness only works in 25% increments.
        if brightness not in [0, 25, 50, 75, 100]:
            msg = f"{brightness}% isn't a valid percentage."
            msg += " Only use 25% increments."
            raise AqualinkInvalidParameterException(msg)

        data = {"aux": self.data["aux"], "light": f"{brightness}"}
        await self.system.set_light(data)


class IaquaColorLight(IaquaAuxSwitch, AqualinkLight):
    async def turn_on(self) -> None:
        if not self.is_on:
            await self.set_effect_by_id(1)

    async def turn_off(self) -> None:
        if self.is_on:
            await self.set_effect_by_id(0)

    @property
    def effect(self) -> str | None:
        # "state"=0 indicates the light is off.
        # "state"=1 indicates the light is on.
        # I don't see a way to retrieve the current effect.
        # The official iAquaLink app doesn't seem to show the current effect
        # choice either, so perhaps it's an unfortunate limitation of the
        # current API.
        return self.data["state"]

    @property
    def supported_effects(self) -> dict[str, int]:
        raise NotImplementedError

    async def set_effect_by_name(self, effect: str) -> None:
        try:
            effect_id = self.supported_effects[effect]
        except KeyError as e:
            msg = f"{effect!r} isn't a valid effect."
            raise AqualinkInvalidParameterException(msg) from e
        await self.set_effect_by_id(effect_id)

    async def set_effect_by_id(self, effect_id: int) -> None:
        try:
            _ = list(self.supported_effects.values()).index(effect_id)
        except ValueError as e:
            msg = f"{effect_id!r} isn't a valid effect."
            raise AqualinkInvalidParameterException(msg) from e

        data = {
            "aux": self.data["aux"],
            "light": str(effect_id),
            "subtype": self.data["subtype"],
        }
        await self.system.set_light(data)


class IaquaColorLightJC(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Jandy"

    @property
    def model(self) -> str:
        return "Colors Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "Alpine White": 1,
            "Sky Blue": 2,
            "Cobalt Blue": 3,
            "Caribbean Blue": 4,
            "Spring Green": 5,
            "Emerald Green": 6,
            "Emerald Rose": 7,
            "Magenta": 8,
            "Garnet Red": 9,
            "Violet": 10,
            "Color Splash": 11,
        }


class IaquaColorLightSL(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Pentair"

    @property
    def model(self) -> str:
        return "SAm/SAL Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "White": 1,
            "Light Green": 2,
            "Green": 3,
            "Cyan": 4,
            "Blue": 5,
            "Lavender": 6,
            "Magenta": 7,
            "Light Magenta": 8,
            "Color Splash": 9,
        }


class IaquaColorLightCL(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Pentair"

    @property
    def model(self) -> str:
        return "ColorLogic Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "Voodoo Lounge": 1,
            "Deep Blue Sea": 2,
            "Afternoon Skies": 3,
            "Emerald": 4,
            "Sangria": 5,
            "Cloud White": 6,
            "Twilight": 7,
            "Tranquility": 8,
            "Gemstone": 9,
            "USA!": 10,
            "Mardi Gras": 11,
            "Cool Cabaret": 12,
        }


class IaquaColorLightJL(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Jandy"

    @property
    def model(self) -> str:
        return "LED WaterColors Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "Alpine White": 1,
            "Sky Blue": 2,
            "Cobalt Blue": 3,
            "Caribbean Blue": 4,
            "Spring Green": 5,
            "Emerald Green": 6,
            "Emerald Rose": 7,
            "Magenta": 8,
            "Violet": 9,
            "Slow Splash": 10,
            "Fast Splash": 11,
            "USA!": 12,
            "Fat Tuesday": 13,
            "Disco Tech": 14,
        }


class IaquaColorLightIB(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Pentair"

    @property
    def model(self) -> str:
        return "Intellibrite Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "SAm": 1,
            "Party": 2,
            "Romance": 3,
            "Caribbean": 4,
            "American": 5,
            "California Sunset": 6,
            "Royal": 7,
            "Blue": 8,
            "Green": 9,
            "Red": 10,
            "White": 11,
            "Magenta": 12,
        }


class IaquaColorLightHU(IaquaColorLight):
    @property
    def manufacturer(self) -> str:
        return "Hayward"

    @property
    def model(self) -> str:
        return "Universal Light"

    @property
    def supported_effects(self) -> dict[str, int]:
        return {
            "Off": 0,
            "Voodoo Lounge": 1,
            "Deep Blue Sea": 2,
            "Royal Blue": 3,
            "Afternoon Skies": 4,
            "Aqua Green": 5,
            "Emerald": 6,
            "Cloud White": 7,
            "Warm Red": 8,
            "Flamingo": 9,
            "Vivid Violet": 10,
            "Sangria": 11,
            "Twilight": 12,
            "Tranquility": 13,
            "Gemstone": 14,
            "USA!": 15,
            "Mardi Gras": 16,
            "Cool Cabaret": 17,
        }


light_subtype_to_class = {
    "1": IaquaColorLightJC,
    "2": IaquaColorLightSL,
    "3": IaquaColorLightCL,
    "4": IaquaColorLightJL,
    "5": IaquaColorLightIB,
    "6": IaquaColorLightHU,
}


class IaquaThermostat(IaquaSwitch, AqualinkThermostat):
    @property
    def _type(self) -> str:
        return self.name.split("_")[0]

    @property
    def _temperature(self) -> str:
        # Spa takes precedence for temp1 if present.
        if self._type == "pool" and "spa_set_point" in self.system.devices:
            return "temp2"
        return "temp1"

    @property
    def unit(self) -> str:
        return self.system.temp_unit

    @property
    def _sensor(self) -> IaquaSensor:
        return cast(IaquaSensor, self.system.devices[f"{self._type}_temp"])

    @property
    def current_temperature(self) -> str:
        return self._sensor.state

    @property
    def target_temperature(self) -> str:
        return self.state

    @property
    def min_temperature(self) -> int:
        if self.unit == "F":
            return IAQUA_TEMP_FAHRENHEIT_LOW
        return IAQUA_TEMP_CELSIUS_LOW

    @property
    def max_temperature(self) -> int:
        if self.unit == "F":
            return IAQUA_TEMP_FAHRENHEIT_HIGH
        return IAQUA_TEMP_CELSIUS_HIGH

    async def set_temperature(self, temperature: int) -> None:
        unit = self.unit
        low = self.min_temperature
        high = self.max_temperature

        if temperature not in range(low, high + 1):
            msg = f"{temperature}{unit} isn't a valid temperature"
            msg += f" ({low}-{high}{unit})."
            raise AqualinkInvalidParameterException(msg)

        data = {self._temperature: str(temperature)}
        await self.system.set_temps(data)

    @property
    def _heater(self) -> IaquaSwitch:
        return cast(IaquaSwitch, self.system.devices[f"{self._type}_heater"])

    @property
    def is_on(self) -> bool:
        return self._heater.is_on

    async def turn_on(self) -> None:
        if self._heater.is_on is False:
            await self._heater.turn_on()

    async def turn_off(self) -> None:
        if self._heater.is_on is True:
            await self._heater.turn_off()



from __future__ import annotations

import logging
import time
from typing import TYPE_CHECKING

from iaqualink.const import MIN_SECS_TO_REFRESH
from iaqualink.exception import (
    AqualinkDeviceNotSupported,
    AqualinkServiceException,
    AqualinkSystemOfflineException,
)
from iaqualink.system import AqualinkSystem
from iaqualink.systems.iaqua.device import IaquaDevice

if TYPE_CHECKING:
    import httpx

    from iaqualink.client import AqualinkClient
    from iaqualink.typing import Payload

IAQUA_SESSION_URL = "https://p-api.iaqualink.net/v1/mobile/session.json"

IAQUA_COMMAND_GET_DEVICES = "get_devices"
IAQUA_COMMAND_GET_HOME = "get_home"
IAQUA_COMMAND_GET_ONETOUCH = "get_onetouch"

IAQUA_COMMAND_SET_AUX = "set_aux"
IAQUA_COMMAND_SET_LIGHT = "set_light"
IAQUA_COMMAND_SET_POOL_HEATER = "set_pool_heater"
IAQUA_COMMAND_SET_POOL_PUMP = "set_pool_pump"
IAQUA_COMMAND_SET_SOLAR_HEATER = "set_solar_heater"
IAQUA_COMMAND_SET_SPA_HEATER = "set_spa_heater"
IAQUA_COMMAND_SET_SPA_PUMP = "set_spa_pump"
IAQUA_COMMAND_SET_TEMPS = "set_temps"


LOGGER = logging.getLogger("iaqualink")


class IaquaSystem(AqualinkSystem):
    NAME = "iaqua"

    def __init__(self, aqualink: AqualinkClient, data: Payload):
        super().__init__(aqualink, data)

        self.temp_unit: str = ""
        self.last_refresh: int = 0

    def __repr__(self) -> str:
        attrs = ["name", "serial", "data"]
        attrs = [f"{i}={getattr(self, i)!r}" for i in attrs]
        return f"{self.__class__.__name__}({' '.join(attrs)})"

    async def _send_session_request(
        self,
        command: str,
        params: Payload | None = None,
    ) -> httpx.Response:
        if not params:
            params = {}

        params.update(
            {
                "actionID": "command",
                "command": command,
                "serial": self.serial,
                "sessionID": self.aqualink.client_id,
            }
        )
        params_str = "&".join(f"{k}={v}" for k, v in params.items())
        url = f"{IAQUA_SESSION_URL}?{params_str}"
        return await self.aqualink.send_request(url)

    async def _send_home_screen_request(self) -> httpx.Response:
        return await self._send_session_request(IAQUA_COMMAND_GET_HOME)

    async def _send_devices_screen_request(self) -> httpx.Response:
        return await self._send_session_request(IAQUA_COMMAND_GET_DEVICES)

    async def update(self) -> None:
        # Be nice to Aqualink servers since we rely on polling.
        now = int(time.time())
        delta = now - self.last_refresh
        if delta < MIN_SECS_TO_REFRESH:
            LOGGER.debug(f"Only {delta}s since last refresh.")
            return

        try:
            r1 = await self._send_home_screen_request()
            r2 = await self._send_devices_screen_request()
        except AqualinkServiceException:
            self.online = None
            raise

        try:
            self._parse_home_response(r1)
            self._parse_devices_response(r2)
        except AqualinkSystemOfflineException:
            self.online = False
            raise

        self.online = True
        self.last_refresh = int(time.time())

    def _parse_home_response(self, response: httpx.Response) -> None:
        data = response.json()

        LOGGER.debug(f"Home response: {data}")

        if data["home_screen"][0]["status"] == "Offline":
            LOGGER.warning(f"Status for system {self.serial} is Offline.")
            raise AqualinkSystemOfflineException

        self.temp_unit = data["home_screen"][3]["temp_scale"]

        # Make the data a bit flatter.
        devices = {}
        for x in data["home_screen"][4:]:
            name = next(iter(x.keys()))
            state = next(iter(x.values()))
            attrs = {"name": name, "state": state}
            devices.update({name: attrs})

        for k, v in devices.items():
            if k in self.devices:
                for dk, dv in v.items():
                    self.devices[k].data[dk] = dv
            else:
                try:
                    self.devices[k] = IaquaDevice.from_data(self, v)
                except AqualinkDeviceNotSupported as e:
                    LOGGER.debug("Device found was ignored: %s", e)

    def _parse_devices_response(self, response: httpx.Response) -> None:
        data = response.json()

        LOGGER.debug(f"Devices response: {data}")

        if data["devices_screen"][0]["status"] == "Offline":
            LOGGER.warning(f"Status for system {self.serial} is Offline.")
            raise AqualinkSystemOfflineException

        # Make the data a bit flatter.
        devices = {}
        for x in data["devices_screen"][3:]:
            aux = next(iter(x.keys()))
            attrs = {"aux": aux.replace("aux_", ""), "name": aux}
            for y in next(iter(x.values())):
                attrs.update(y)
            devices.update({aux: attrs})

        for k, v in devices.items():
            if k in self.devices:
                for dk, dv in v.items():
                    self.devices[k].data[dk] = dv
            else:
                try:
                    self.devices[k] = IaquaDevice.from_data(self, v)
                except AqualinkDeviceNotSupported as e:
                    LOGGER.info("Device found was ignored: %s", e)

    async def set_switch(self, command: str) -> None:
        r = await self._send_session_request(command)
        self._parse_home_response(r)

    async def set_temps(self, temps: Payload) -> None:
        # I'm not proud of this. If you read this, please submit a PR to make it better.
        # We need to pass the temperatures for both pool and spa (if present) in the same request.
        # Set args to current target temperatures and override with the request payload.
        args = {}
        i = 1
        if "spa_set_point" in self.devices:
            args[f"temp{i}"] = self.devices["spa_set_point"].target_temperature
            i += 1
        args[f"temp{i}"] = self.devices["pool_set_point"].target_temperature
        args.update(temps)

        r = await self._send_session_request(IAQUA_COMMAND_SET_TEMPS, args)
        self._parse_home_response(r)

    async def set_aux(self, aux: str) -> None:
        aux = IAQUA_COMMAND_SET_AUX + "_" + aux.replace("aux_", "")
        r = await self._send_session_request(aux)
        self._parse_devices_response(r)

    async def set_light(self, data: Payload) -> None:
        r = await self._send_session_request(IAQUA_COMMAND_SET_LIGHT, data)
        self._parse_devices_response(r)



from __future__ import annotations

import unittest
from unittest.mock import MagicMock, patch

import pytest

from iaqualink.client import AqualinkClient
from iaqualink.exception import AqualinkSystemUnsupportedException
from iaqualink.system import AqualinkSystem


class TestAqualinkSystem(unittest.IsolatedAsyncioTestCase):
    def setUp(self) -> None:
        pass

    def test_repr(self) -> None:
        aqualink = MagicMock()
        data = {
            "id": 1,
            "serial_number": "ABCDEFG",
            "device_type": "iaqua",
            "name": "foo",
        }
        system = AqualinkSystem(aqualink, data)
        assert (
            repr(system)
            == f"AqualinkSystem(name='foo', serial='ABCDEFG', data={data})"
        )

    def test_from_data_iaqua(self) -> None:
        aqualink = MagicMock()
        data = {"id": 1, "serial_number": "ABCDEFG", "device_type": "iaqua"}
        r = AqualinkSystem.from_data(aqualink, data)
        assert r is not None

    def test_from_data_unsupported(self) -> None:
        aqualink = MagicMock()
        data = {"id": 1, "serial_number": "ABCDEFG", "device_type": "foo"}
        with pytest.raises(AqualinkSystemUnsupportedException):
            AqualinkSystem.from_data(aqualink, data)

    async def test_get_devices_needs_update(self) -> None:
        data = {"id": 1, "serial_number": "ABCDEFG", "device_type": "fake"}
        aqualink = AqualinkClient("user", "pass")
        system = AqualinkSystem(aqualink, data)
        system.devices = None

        with patch.object(system, "update") as mock_update:
            await system.get_devices()
            mock_update.assert_called_once()

    async def test_get_devices(self) -> None:
        data = {"id": 1, "serial_number": "ABCDEFG", "device_type": "fake"}
        aqualink = AqualinkClient("user", "pass")
        system = AqualinkSystem(aqualink, data)
        system.devices = {"foo": "bar"}

        with patch.object(system, "update") as mock_update:
            await system.get_devices()
            mock_update.assert_not_called()

    async def test_update_not_implemented(self) -> None:
        data = {"id": 1, "serial_number": "ABCDEFG", "device_type": "fake"}
        aqualink = AqualinkClient("user", "pass")
        system = AqualinkSystem(aqualink, data)

        with pytest.raises(NotImplementedError):
            await system.update()



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, ClassVar

from iaqualink.exception import AqualinkSystemUnsupportedException

if TYPE_CHECKING:
    from iaqualink.client import AqualinkClient
    from iaqualink.device import AqualinkDevice
    from iaqualink.typing import Payload


LOGGER = logging.getLogger("iaqualink")


class AqualinkSystem:
    subclasses: ClassVar[dict[str, type[AqualinkSystem]]] = {}

    def __init__(self, aqualink: AqualinkClient, data: Payload):
        self.aqualink = aqualink
        self.data = data
        self.devices: dict[str, AqualinkDevice] = {}
        self.last_refresh: int

        # Semantics here are somewhat odd.
        # True/False are obvious, None means "unknown".
        self.online: bool | None = None

    @classmethod
    def __init_subclass__(cls) -> None:
        super().__init_subclass__()
        if hasattr(cls, "NAME"):
            cls.subclasses[cls.NAME] = cls

    def __repr__(self) -> str:
        attrs = ["name", "serial", "data"]
        attrs = [f"{i}={getattr(self, i)!r}" for i in attrs]
        return f"{self.__class__.__name__}({', '.join(attrs)})"

    @property
    def name(self) -> str:
        return self.data["name"]

    @property
    def serial(self) -> str:
        return self.data["serial_number"]

    @classmethod
    def from_data(
        cls, aqualink: AqualinkClient, data: Payload
    ) -> AqualinkSystem:
        if data["device_type"] not in cls.subclasses:
            m = f"{data['device_type']} is not a supported system type."
            LOGGER.warning(m)
            raise AqualinkSystemUnsupportedException(m)

        return cls.subclasses[data["device_type"]](aqualink, data)

    async def get_devices(self) -> dict[str, AqualinkDevice]:
        if not self.devices:
            await self.update()
        return self.devices

    async def update(self) -> None:
        raise NotImplementedError



import copy

import httpx
import pytest
import respx.router

from iaqualink.exception import (
    AqualinkServiceException,
    AqualinkServiceUnauthorizedException,
)

from .base import TestBase, dotstar, resp_200


class TestBaseSystem(TestBase):
    def test_propery_name(self) -> None:
        assert isinstance(self.sut.name, str)

    def test_property_serial(self) -> None:
        assert isinstance(self.sut.name, str)

    def test_from_data(self) -> None:
        if sut_class := getattr(self, "sut_class", None):
            assert isinstance(self.sut, sut_class)

    @respx.mock
    async def test_update_success(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.update()
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_update_consecutive(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.update()
        respx_mock.reset()
        await self.sut.update()
        assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_update_service_exception(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        resp_500 = httpx.Response(status_code=500)
        respx_mock.route(dotstar).mock(resp_500)
        with pytest.raises(AqualinkServiceException):
            await self.sut.update()
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_update_request_unauthorized(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        resp_401 = httpx.Response(status_code=401)
        respx_mock.route(dotstar).mock(resp_401)
        with pytest.raises(AqualinkServiceUnauthorizedException):
            await self.sut.update()
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)

    @respx.mock
    async def test_get_devices(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        self.sut.devices = {"foo": {}}
        await self.sut.get_devices()
        assert len(respx_mock.calls) == 0

    @respx.mock
    async def test_get_devices_needs_update(
        self, respx_mock: respx.router.MockRouter
    ) -> None:
        respx_mock.route(dotstar).mock(resp_200)
        await self.sut.get_devices()
        assert len(respx_mock.calls) > 0
        self.respx_calls = copy.copy(respx_mock.calls)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

from iaqualink.exception import AqualinkOperationNotSupportedException

if TYPE_CHECKING:
    from iaqualink.typing import DeviceData

LOGGER = logging.getLogger("iaqualink")


class AqualinkDevice:
    def __init__(
        self,
        system: Any,  # Should be AqualinkSystem but causes mypy errors.
        data: DeviceData,
    ):
        self.system = system
        self.data = data

    def __repr__(self) -> str:
        attrs = ["data"]
        attrs = [f"{i}={getattr(self, i)!r}" for i in attrs]
        return f"{self.__class__.__name__}({', '.join(attrs)})"

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, AqualinkDevice):
            return NotImplemented

        if (
            self.system.serial == other.system.serial
            and self.data == other.data
        ):
            return True
        return False

    @property
    def label(self) -> str:
        raise NotImplementedError

    @property
    def state(self) -> str:
        raise NotImplementedError

    @property
    def name(self) -> str:
        raise NotImplementedError

    @property
    def manufacturer(self) -> str:
        raise NotImplementedError

    @property
    def model(self) -> str:
        raise NotImplementedError


class AqualinkSensor(AqualinkDevice):
    pass


class AqualinkBinarySensor(AqualinkSensor):
    """These are non-actionable sensors, essentially read-only on/off."""

    @property
    def is_on(self) -> bool:
        raise NotImplementedError


class AqualinkSwitch(AqualinkBinarySensor, AqualinkDevice):
    async def turn_on(self) -> None:
        raise NotImplementedError

    async def turn_off(self) -> None:
        raise NotImplementedError


class AqualinkLight(AqualinkSwitch, AqualinkDevice):
    @property
    def brightness(self) -> int | None:
        return None

    @property
    def supports_brightness(self) -> bool:
        return self.brightness is not None

    async def set_brightness(self, _: int) -> None:
        if self.supports_brightness is True:
            raise NotImplementedError
        raise AqualinkOperationNotSupportedException

    @property
    def effect(self) -> str | None:
        return None

    @property
    def supports_effect(self) -> bool:
        return self.effect is not None

    async def set_effect_by_name(self, _: str) -> None:
        if self.supports_effect is True:
            raise NotImplementedError
        raise AqualinkOperationNotSupportedException

    async def set_effect_by_id(self, _: int) -> None:
        if self.supports_effect is True:
            raise NotImplementedError
        raise AqualinkOperationNotSupportedException


class AqualinkThermostat(AqualinkSwitch, AqualinkDevice):
    @property
    def unit(self) -> str:
        raise NotImplementedError

    @property
    def current_temperature(self) -> str:
        raise NotImplementedError

    @property
    def target_temperature(self) -> str:
        raise NotImplementedError

    @property
    def max_temperature(self) -> int:
        raise NotImplementedError

    @property
    def min_temperature(self) -> int:
        raise NotImplementedError

    async def set_temperature(self, _: int) -> None:
        raise NotImplementedError



from __future__ import annotations

from unittest.mock import MagicMock, patch

import pytest

from iaqualink.exception import (
    AqualinkServiceUnauthorizedException,
    AqualinkSystemOfflineException,
)
from iaqualink.system import AqualinkSystem
from iaqualink.systems.iaqua.device import IaquaAuxSwitch
from iaqualink.systems.iaqua.system import IaquaSystem

from ...base_test_system import TestBaseSystem


class TestIaquaSystem(TestBaseSystem):
    def setUp(self) -> None:
        super().setUp()

        data = {
            "id": 123456,
            "serial_number": "SN123456",
            "created_at": "2017-09-23T01:00:08.000Z",
            "updated_at": "2017-09-23T01:00:08.000Z",
            "name": "Pool",
            "device_type": "iaqua",
            "owner_id": None,
            "updating": False,
            "firmware_version": None,
            "target_firmware_version": None,
            "update_firmware_start_at": None,
            "last_activity_at": None,
        }
        self.sut = AqualinkSystem.from_data(self.client, data=data)
        self.sut_class = IaquaSystem

    async def test_update_success(self) -> None:
        with (
            patch.object(self.sut, "_parse_home_response"),
            patch.object(self.sut, "_parse_devices_response"),
        ):
            await super().test_update_success()

    async def test_update_offline(self) -> None:
        with patch.object(self.sut, "_parse_home_response") as mock_parse:
            mock_parse.side_effect = AqualinkSystemOfflineException
            with pytest.raises(AqualinkSystemOfflineException):
                await super().test_update_success()
            assert self.sut.online is False

    async def test_update_consecutive(self) -> None:
        with (
            patch.object(self.sut, "_parse_home_response"),
            patch.object(self.sut, "_parse_devices_response"),
        ):
            await super().test_update_consecutive()

    async def test_get_devices_needs_update(self) -> None:
        with (
            patch.object(self.sut, "_parse_home_response"),
            patch.object(self.sut, "_parse_devices_response"),
        ):
            await super().test_get_devices_needs_update()

    async def test_parse_devices_offline(self) -> None:
        message = {"message": "", "devices_screen": [{"status": "Offline"}]}
        response = MagicMock()
        response.json.return_value = message

        with pytest.raises(AqualinkSystemOfflineException):
            self.sut._parse_devices_response(response)
        assert self.sut.devices == {}

    async def test_parse_devices_good(self) -> None:
        message = {
            "message": "",
            "devices_screen": [
                {"status": "Online"},
                {"response": ""},
                {"group": "1"},
                {
                    "aux_B1": [
                        {"state": "0"},
                        {"label": "Label B1"},
                        {"icon": "aux_1_0.png"},
                        {"type": "0"},
                        {"subtype": "0"},
                    ]
                },
            ],
        }
        response = MagicMock()
        response.json.return_value = message

        expected = {
            "aux_B1": IaquaAuxSwitch(
                system=self.sut,
                data={
                    "aux": "B1",
                    "name": "aux_B1",
                    "state": "0",
                    "label": "Label B1",
                    "icon": "aux_1_0.png",
                    "type": "0",
                    "subtype": "0",
                },
            )
        }
        self.sut._parse_devices_response(response)
        assert self.sut.devices == expected

    @patch("httpx.AsyncClient.request")
    async def test_home_request(self, mock_request) -> None:
        mock_request.return_value.status_code = 200

        await self.sut._send_home_screen_request()

    @patch("httpx.AsyncClient.request")
    async def test_home_request_unauthorized(self, mock_request) -> None:
        mock_request.return_value.status_code = 401

        with pytest.raises(AqualinkServiceUnauthorizedException):
            await self.sut._send_home_screen_request()

    @patch("httpx.AsyncClient.request")
    async def test_devices_request(self, mock_request) -> None:
        mock_request.return_value.status_code = 200

        await self.sut._send_devices_screen_request()

    @patch("httpx.AsyncClient.request")
    async def test_devices_request_unauthorized(self, mock_request) -> None:
        mock_request.return_value.status_code = 401

        with pytest.raises(AqualinkServiceUnauthorizedException):
            await self.sut._send_devices_screen_request()
